@@ -17,6 +17,7 @@ GNU General Public License for more details.
 '''
 
 import re
+import asyncio
 from time import sleep
 from playwright.sync_api import sync_playwright
 from playwright.async_api import async_playwright
@@ -27,7 +28,7 @@ from fake_useragent import UserAgent
 class AsyncObject(object):
     """Inheriting this class allows me to define an async
     __init__, So you can create an async Cleverbot object
-    by doing `await Cleverbot(p_w)`"""
+    by doing `await CleverbotAsync(p_w)`"""
     async def __new__(cls, *a, **kw):
         instance = super().__new__(cls)
         await instance.__init__(*a, **kw)
@@ -46,66 +47,75 @@ class Cleverbot():
     def __init__(self, p_w: object):
         """ Initialize playwright and connect to cleverbot.com."""
         self.p_w: object = p_w
-        self.url: str = 'https://www.cleverbot.com'
+        self.url: str = "https://www.cleverbot.com"
         self.hacking: bool = False
+        self.bot_response: str = ""
         self.browser: object = self.p_w.firefox.launch()
-        self.page: object = self.browser.new_page(
+        self.context: object = self.browser.new_context(
             user_agent=UserAgent().random)
+
+    def get_form(self):
+        """Open new browser context page."""
         while True:
             try:
+                self.page: object = self.context.new_page()
                 self.page.goto(self.url,
                                timeout=10000,
                                wait_until="domcontentloaded")
             except (PwTimeout, BrokenPipeError):
+                self.page.close()
                 continue
             break
 
     def send_input(self, user_input: str):
         """ Submits your message through an input filter."""
-        f_one: str = r'<\/?[a-z]+>|<DOCTYPE'
-        f_two: str = r'/<[^>]+>/g'
+        f_one: str = r"<\/?[a-z]+>|<DOCTYPE"
+        f_two: str = r"/<[^>]+>/g"
         if re.search(f_one, user_input) is not None or re.search(
                 f_two, user_input) is not None:
             self.hacking: bool = True
-            user_input: str = 'I will hack you'
+            user_input: str = "I will hack you"
         while True:
             try:
                 self.page.evaluate(f'cleverbot.sendAI("{user_input}")')
+                sleep(0.25)
             except BrokenPipeError:
                 continue
             break
 
+    def _parse_response(self, response: object):
+        """Helper method to parse network responses for reply."""
+        if "talking.txt" in response.url:
+            cookies: str = response.request.headers["cookie"]
+            cookie_list: list = cookies.split("; ")
+            for cookie in cookie_list:
+                if cookie.startswith("CBALT"):
+                    self.bot_response: str = cookie[8:]
+
     def get_response(self) -> str:
         """The DOM is updated with every individual character
-        received from the Cleverbot app. This tries to make
-        sure that the DOM element has receive the full text
-        before continuing the function."""
-        while self.hacking is False:
-            try:
-                while True:
-                    sleep(1)
-                    line: str = self.page.text_content('id=line1')
-                    sleep(3)
-                    new_line: str = self.page.text_content('id=line1')
-                    if line != new_line and new_line != '':
-                        line: str = self.page.text_content('id=line1')
-                        sleep(3)
-                        break
-            except BrokenPipeError:
-                continue
-            break
+        received from the Cleverbot app. This captures network
+        requests made after sending input, tries to make sure
+        that the DOM element has received text then retrieves
+        the reply."""
         if self.hacking is True:
-            bot_response: str = 'Silly rabbit, html is for skids.'
-        elif self.hacking is False:
-            bot_response: str = line
-        self.hacking: bool = False
-        return bot_response
+            self.hacking: bool = False
+            return "No hax bro."
+        self.page.on("response",
+                     lambda response: self._parse_response(response))
+        line: str = self.page.text_content("id=line1")
+        while len(line) <= 1 and line != self.bot_response:
+            line: str = self.page.text_content("id=line1")
+            sleep(0.1)
+        self.page.close()
+        return self.bot_response
 
     def single_exchange(self, user_input: str) -> str:
         """This fuction is used to create a single send a receive chat
         session via a headless Firefox browser, sending your input
         as an argument to the cleverbot.sendAI() function and
-        retrieving it's response from the DOM."""
+        retrieving it's reply."""
+        self.get_form()
         self.send_input(user_input)
         return self.get_response()
 
@@ -121,63 +131,73 @@ class CleverbotAsync(AsyncObject):
         self.p_w: object = p_w
         self.url: str = 'https://www.cleverbot.com'
         self.hacking: bool = False
+        self.bot_response: str = ""
         self.browser: object = await self.p_w.firefox.launch()
-        self.page: object = await self.browser.new_page(
+        self.context: object = await self.browser.new_context(
             user_agent=UserAgent().random)
+
+    async def get_form(self):
+        """Open new browser context page."""
         while True:
             try:
+                self.page: object = await self.context.new_page()
                 await self.page.goto(self.url,
                                      timeout=10000,
                                      wait_until="domcontentloaded")
             except (PwTimeout, BrokenPipeError):
+                await self.page.close()
                 continue
             break
 
     async def send_input(self, user_input: str):
         """ Submits your message through an input filter."""
-        f_one: str = r'<\/?[a-z]+>|<DOCTYPE'
-        f_two: str = r'/<[^>]+>/g'
+        f_one: str = r"<\/?[a-z]+>|<DOCTYPE"
+        f_two: str = r"/<[^>]+>/g"
         if re.search(f_one, user_input) is not None or re.search(
                 f_two, user_input) is not None:
             self.hacking: bool = True
-            user_input: str = 'I will hack you'
+            user_input: str = "I will hack you"
         while True:
             try:
                 await self.page.evaluate(f'cleverbot.sendAI("{user_input}")')
+                await asyncio.sleep(0.25)
             except BrokenPipeError:
                 continue
             break
 
+    def _parse_response(self, response: object):
+        """Helper method to parse network responses for reply."""
+        if "talking.txt" in response.url:
+            cookies: str = response.request.headers["cookie"]
+            cookie_list: list = cookies.split("; ")
+            for cookie in cookie_list:
+                if cookie.startswith("CBALT"):
+                    self.bot_response: str = cookie[8:]
+
     async def get_response(self) -> str:
         """The DOM is updated with every individual character
-        received from the Cleverbot app. This tries to make
-        sure that the DOM element has receive the full text
-        before continuing the function."""
-        while self.hacking is False:
-            try:
-                while True:
-                    sleep(1)
-                    line: str = await self.page.text_content('id=line1')
-                    sleep(3)
-                    new_line: str = await self.page.text_content('id=line1')
-                    if line != new_line and new_line != '':
-                        line: str = await self.page.text_content('id=line1')
-                        sleep(3)
-                        break
-            except BrokenPipeError:
-                continue
-            break
+        received from the Cleverbot app. This captures network
+        requests made after sending input, tries to make sure
+        that the DOM element has received text then retrieves
+        the reply."""
         if self.hacking is True:
-            bot_response: str = 'Silly rabbit, html is for skids.'
-        elif self.hacking is False:
-            bot_response: str = line
-        self.hacking: bool = False
-        return bot_response
+            self.hacking: bool = False
+            return "No hax bro."
+        self.page.on("response",
+                     lambda response: self._parse_response(response))
+        line: str = await self.page.text_content("id=line1")
+        while len(line) <= 1 and line != self.bot_response:
+            line: str = await self.page.text_content("id=line1")
+            await asyncio.sleep(0.1)
+        await self.page.close()
+
+        return self.bot_response
 
     async def single_exchange(self, user_input: str) -> str:
         """This fuction is used to create a single send a receive chat
         session via a headless Firefox browser, sending your input
         as an argument to the cleverbot.sendAI() function and
-        retrieving it's response from the DOM."""
+        retrieving it's reply."""
+        await self.get_form()
         await self.send_input(user_input)
         return await self.get_response()
